import string
from cStringIO import StringIO

import OFS.SimpleItem             
import Acquisition                
import AccessControl.Role         

from Globals import Persistent
from Globals import HTMLFile

class SimpleCache(
    OFS.SimpleItem.Item,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager
    ):
    """A simple cache is a simple dependency based cache. If any of the
    dependent objects has a changed timestamp, the cache will be automatically
    be updated from the producer method when queried.
    """
    meta_type = "SimpleCache"
    
    manage_options = (
        {'label' : 'Edit',     'action' : 'manage_editForm'},
        {'label' : 'View',     'action' : 'view'},
        {'label' : 'Security', 'action' : 'manage_access'},
        )
    
    __ac_permissions__ = (
        ('View management screens',
         ('manage_editForm',
          'manage_tabs',)),

        ('Change SimpleCaches',
         ('manage_edit',
          'update',)),
        
        ('Change permissions',
         ('manage_access',)),

        ('Access contents information',
         ('showDependencies',)),

        ('View',
         ('view',)),        
        )

    manage_editForm = HTMLFile('cacheEdit', globals())
    
    def __init__(self):
        self.producer = ""
        self.dependencies = []
        self.data = ""
        self.debug = 0
        
    def manage_edit(self, REQUEST):
        """Edit the dependencies. Also updates the cache.
        """
        # update producer
        self.producer = string.strip(REQUEST['producer'])
        
        # update dependencies
        dependencies = []

        for dependency in REQUEST['dependencies']:
            dependency = string.strip(dependency)
            if dependency == "":
                continue
            dependencies.append(dependency)

        self.dependencies = dependencies

        if REQUEST.has_key('debug'):
            self.debug = 1
        else:
            self.debug = 0
            
        # force update cache
        self.update(REQUEST)

        # done with editing
        if REQUEST is not None:
            message = "Cache changed"
            return self.manage_editForm(self, REQUEST,
                                        manage_tabs_message=message)

    def update(self, REQUEST):
        """Update the cache.
        """
        folder = self.aq_parent
        # complain if producer does not exist in folder
        if not hasattr(folder, self.producer):
            raise Error, "Producer %s does not exist." % self.producer
        self.data = getattr(folder, self.producer)(folder, REQUEST)

    def _need_update(self):
        """Return 1 if an update of the cache is needed, 0 otherwise.
        """
        cache_timestamp = self.bobobase_modification_time()
        folder = self.aq_parent
        
        dependencies = [self.producer] + self.dependencies
    
        for dependency in dependencies:
            timestamp = getattr(folder, dependency).\
                        bobobase_modification_time()
            if timestamp > cache_timestamp:
                return 1
            
        # don't need an update, as timestamps of dependencies are older
        return 0

    def view(self, REQUEST):
        """Access/view cache. Update if necessary.
        """
        # update cache if necessary
        if self._need_update():
            self.update(REQUEST)
            return self.data
        else:
            # return data
            if self.debug:
                return "[cached by '%s'] %s" % (self.id, self.data)
            else:
                return self.data

    index_html = view
    
    def showDependencies(self):
        """Show all dependencies for this cache.
        """
        result = StringIO()
        for dependency in self.dependencies:
            result.write(dependency)
            result.write("\n")
        return result.getvalue()

class Error(Exception):
    pass

# form to add the cache to a folder
manage_addSimpleCacheForm = HTMLFile('cacheAdd', globals())

# adds cache to folder
def manage_addSimpleCache(self, id, title='', REQUEST=None):
    """Add cache to folder.
    """
    cache = SimpleCache()
    cache.id = id

    self._setObject(id, cache)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)






